/*********************************************************************
  zupf.cc -- Modellierung einer gezupften Saite
  Autor    : R. Hilfer
  Zweck    : Beispiel zur Vorlesung, demonstriert Karplus-Strong Alg
  Gebrauch : Eingabe von stdin:
               frq = Frequenz in Hertz
               t_max = Gesamtzeit
   Version  : 1.0 (Originalversion) erstellt am  12.10.2000
   
   modifiziert für Ausgabe auf soundkarte (linux)  myr 1.4.2014
   g++ -Wall -o zupf zupf.cpp
   Gebrauch: 
   ./zupf      dann eingabe von stdin
   oder
   ./zupf frequenz[Hz] gesamtzeit[sec]  z.B.  ./zupf 440 5
 *********************************************************************/

// Einbinden benoetigter Bibliotheken
#include <iostream>
#include <vector>

#include <stdio.h>
#include <stdlib.h>
# include <unistd.h>
# include <fcntl.h>
# include <sys/ioctl.h>
# include <linux/soundcard.h>

using namespace std;
int main(int argc, char *argv[])
{
// Deklarationen
   double *u;
   vector<short> out;
   double u_0,uprv,t_max,dt,t,frq;
   short amp;
   int plckpt,len,i;
   int fd, format=AFMT_S16_LE;
   int j;

// /dev/dsp vorbereiten   
   if ((fd = open("/dev/dsp", O_RDWR)) == -1)
   {
    perror("rec_play: Can't open device");
    return(1);
   }

  if (ioctl(fd, SNDCTL_DSP_SETFMT, &format) == -1)
    perror(" Can't set format");

  i = 0;
  if (ioctl(fd, SNDCTL_DSP_STEREO, &i) == -1)
    perror(" Can't set to mono");

  i = 44100;
  if (ioctl(fd, SNDCTL_DSP_SPEED, &i) == -1)
    perror(" Can't set sampling rate");    

// Parameter abfragen und einlesen
   if(argc == 3)
   {
      sscanf(argv[1],"%lf",&frq);
      sscanf(argv[2],"%lf",&t_max);
   }
   else
   {
     cout << "Frequenz      :";  cin >> frq;
     cout << "Gesamtdauer   :";  cin >> t_max;
   }
   
   dt=1./44100.;             // Zeitschritt = Abtastrate
   len = int(44100./frq);    // Laenge der Saite
   plckpt = int(0.1*len);    // Zupfpunkt
   u_0 = 28000;              // Anfangsauslenkung
   t=0.;                     // Zeit initialisieren

 
// Initialisieren der Saite
   u = new double[len];             // Speicher bereitstellen
   for( i=0; i<=plckpt; i++ ){      // Auffuellen mit Saegezahn
     u[i]=2*u_0*i/plckpt - u_0;
   }
   for( i=len-1; i>plckpt; i-- ){
     u[i]=2*u_0*(len-1-i)/(len-plckpt) - u_0;
   }

//	for(int i = 0; i < len; i++)
//	{
//		cout << u[i] << endl;
//	}

// process
   i=0;
   j=0;
   uprv=u[0];
   while(t<t_max){
     u[i] = u[i]*0.5+uprv*0.5;  // Tiefpass = gleitender Mittelwert
     uprv = u[i];               // Tiefpass: Amplitude zwischenspeichern
     amp = short(u[i]);         // Vorbereitung zur Ausgabe in short
//
//     cout << amp << endl;
//
     out.push_back(amp); 		// Sample an vector anhängen
     t=t+dt;                    // Zeit hochzaehlen
     if(i==len-1) i=-1;         // Zaehler zuruecksetzen (ringbuffer)
     i++; 						// Index fuer naechste Ausgabe setzen
     j++; 						// Anzahl der Samples mitzählen                     
   }
   
   if(write(fd, &out[0], j)==-1)// Samples -> Soundkarte
     perror(" Can't play audio data");

// finish
   close(fd);  
   return 0;
}

